"""
    This file was generated by flowtool-versioning.
"""

VERSION_INFO = {'vcs_info': {'commit': 'fa1d22c4e8474a658ab29c75a389818be5db43f9',
              'dirt': '',
              'latest_tag': 'flowtool-releasing-0.7.7',
              'latest_tag_commit': 'fa1d22c4e8474a658ab29c75a389818be5db43f9',
              'latest_tag_version': '0.7.7',
              'prefix': 'flowtool-releasing-',
              'prefix_tag_distances': {'flowtool-releasing-0.7.5': 5,
                                       'flowtool-releasing-0.7.6': 4,
                                       'flowtool-releasing-0.7.7': 0},
              'tag_version': {'normalized': '0.7.7',
                              'release': (0, 7, 7),
                              'version': '0.7.7'}},
 'version': '0.7.7'}

def get_version():
    global VERSION_INFO
    return VERSION_INFO.get('version', '$Format:%d %H$')
